clear;    %clear all variables in memory

%----------------INPUT PARAMETERS: ----------------------
fid = fopen('Parameters.txt','r');  %input file  *.in  with the initial LITE parameters, only for reading
row=0;
i=0;
while row ~= -1
    i=i+1;
    row=fgets(fid);
    if i == 1
        BBB = strread(row, '%s', 'delimiter', ' ');
        Star = cellstr([BBB(1)]);
%         Star = cellstr([BBB(1) BBB(2) BBB(3) BBB(4) BBB(5) BBB(6) BBB(7) BBB(8) BBB(9) BBB(10)]); 
        Star = Star';
    else
       if row ~= -1
         A=sscanf(row,'%f');
         Y(i-1)=A(1);
         F(i-1)=A(2);
       end
    end
end
status = fclose('all');      %close the file
clear A row i fid status BBB;    %it is better to clear all unnecessary variables from the memory
Y=Y';
%------------------------------------------------------------

%---------------INPUT DATA POINTS (=minima times): ---------------------
fid2 = fopen('Minima.txt','r');  %input file  *.min  with the data (times of minima - JDi), open only for reading
row=0;
i=0;
while row ~= -1
    i=i+1;
    row=fgets(fid2);
    if i ~= 1
       if row ~= -1
         A=sscanf(row,'%f');
         Minima(i-1)=A(1);
         Weight(i-1)=A(2);
     end
    end
end
status = fclose('all');
clear A row fid2 status;
%----------------------------------------------------------------
N=i-2;      %number of data points (minimum times)
param=5;     %number of parameters to fit (5 apsidal parameters)

alfa=1.0;     %parameters of the simplex itself:
beta=0.35;
gama=2.0;
delta=0.5;

JD00=Y(1);    %here are the 5 parameters of the Apsidal motion from the Y matrix
JD0F=F(1);
P=Y(2);
PF=F(2);
excen1=Y(3);
excen1F=F(3);
omega1=Y(4);
omega1F=F(4);
dodt=Y(5);
dodtF=F(5);
clear Y;

Minima=Minima';     %minima times
Weight=Weight';      %weights of individual minima times
JDi=Minima+2400000;   %Juliane date of individual minima times

Epo0=Epoch(JDi,JD00,P);    %Epoch of individual minima times

O=JDi-JD00-P*Epo0;         %O-C difference

NumberPrim=0;
NumberSec=0;
for i = 1:N
   switch round(10*abs(Epo0(i)-round(Epo0(i))))
   case 0           %primary minimum times
       NumberPrim=NumberPrim+1;
       E11(NumberPrim)=Epo0(i);
       Occ1(NumberPrim)=O(i);
       B1(NumberPrim)=Weight(i);
   case 5            %secondary minimum times
       NumberSec=NumberSec+1;
       E21(NumberSec)=Epo0(i);
       Occ2(NumberSec)=O(i);   
       B2(NumberSec)=Weight(i);
   end
end

m=1;
n=1;
o=1;
o1=1;
remove=0;       %some minimum times were eliminated
for i = 1:NumberPrim     %primary minimum times
   switch B1(i)
   case 0          %weight = 0  =>  ignore this minimum!
      remove=remove+1;     %removed minimum times (=weight 0)
   case 1        %visual primary minimum times
      E1v(n)=E11(i);
      O1v(n)=Occ1(i);
      n=n+1;
   case 2        %visual primary minimum times
      E1v(n)=E11(i);
      O1v(n)=Occ1(i);
      n=n+1;
   case 3        %visual primary minimum times
      E1v(n)=E11(i);
      O1v(n)=Occ1(i);
      n=n+1;
   case 4        %visual primary minimum times
      E1v(n)=E11(i);
      O1v(n)=Occ1(i);
      n=n+1;
   case 5       %photographic primary minimum times, worse quality
      E1pp(o1)=E11(i);
      O1pp(o1)=Occ1(i);
      o1=o1+1;
   case 10        %photographic primary minimum times
      E1p(o)=E11(i);
      O1p(o)=Occ1(i);
      o=o+1;
   case 30       %ccd primary minimum times
      E1c(m)=E11(i);
      O1c(m)=Occ1(i);
      m=m+1;
   end
end

p=1;
q=1;
r=1;
o2=1;
for i = 1:NumberSec     %secondary minimum times
   switch B2(i)
   case 0
      remove=remove+1;   %removed secondary minimum times
   case 1         %visual secondary minimum times
      E2v(p)=E21(i);
      O2v(p)=Occ2(i);
      p=p+1;
   case 2         %visual secondary minimum times
      E2v(p)=E21(i);
      O2v(p)=Occ2(i);
      p=p+1;
   case 3         %visual secondary minimum times
      E2v(p)=E21(i);
      O2v(p)=Occ2(i);
      p=p+1;
   case 4         %visual secondary minimum times
      E2v(p)=E21(i);
      O2v(p)=Occ2(i);
      p=p+1;
   case 5       %photographic secondary minimum times, worse quality
      E2pp(o2)=E21(i);
      O2pp(o2)=Occ2(i);
      o2=o2+1;
   case 10         %photographic secondary minimum times
      E2p(q)=E21(i);
      O2p(q)=Occ2(i);
      q=q+1;
   case 30        %ccd secondary minimum times
      E2c(r)=E21(i);
      O2c(r)=Occ2(i);
      r=r+1;
   end
end

% %--------------- HERE PLOT THE FIGURE: --------------------
% figure(1);
% clf;
% if n > 1 plot(E1v,O1v,'k.','MarkerSize',6); end    %plot primary visual minimum times
% hold on
% if o1 > 1 plot(E1pp,O1pp,'k.','MarkerSize',8); end  %primary photographic minimum times, worse quality
% if o > 1 plot(E1p,O1p,'k.','MarkerSize',12); end   %primary photographic minimum times
% if m > 1 plot(E1c,O1c,'k.','MarkerSize',17); end   %primary ccd minimum times
% if p > 1 plot(E2v,O2v,'bo','MarkerSize',2); end    %secondary visual minimum times
% if o2 > 1 plot(E2pp,O2pp,'bo','MarkerSize',2); end  %secondary photographic minimum times, worse quality
% if q > 1 plot(E2p,O2p,'bo','MarkerSize',3); end    %secondary photographic minimum times
% if r > 1 plot(E2c,O2c,'bo','MarkerSize',4); end    %secondary ccd minimum times
% xlabel('Epoch');   %labeling the axis
% ylabel('O-C (Days)');
% title('O-C diagram YYSgr');   %labeling the figure
% hold off
%--------------------------------------------------------------------------
% clear E1c E1v E1p E2c E2v E2p E1pp E2pp O1pp O2pp O1v O1c O1p O2v O2c O2p j k m n o o1 p q r o2 B1 B2 Occ1 Occ2 E11 E21 O;   %clear all unnecessary variables
%---------------------------------------------------------------------------------------------------------
minimumE=1000*floor(min(Epo0)/1000)-00;
maximumE=1000*ceil(max(Epo0)/1000)+800;

i=0;
Pa=P/(1-dodt*pi/(2*pi*180));
for Ep = minimumE:10:maximumE
    i=i+1;
    Epo(i) = Ep;
    Tprim(i) = Primary(JD00,Ep,P,excen1,dodt*pi/180,omega1*pi/180)-JD00-P*Ep;
    Tsec(i)  = Secondary(JD00,Ep,P,excen1,dodt*pi/180,omega1*pi/180)-JD00-P*Ep-Pa/2;
end
clear i Ep;

CC=['O-C diagram '];
BB=char(Star);
QQ=sprintf([CC BB]);

figure(2);
clf;
hold on
plot(Epo,Tprim,'k-');
plot(Epo,Tsec,'b-');
set(gca,'box','on');
if n > 1 plot(E1v,O1v,'k.','MarkerSize',6); end    %plot primary visual minimum times
if o1 > 1 plot(E1pp,O1pp,'k.','MarkerSize',8); end  %primary photographic minimum times, worse quality
if o > 1 plot(E1p,O1p,'k.','MarkerSize',12); end   %primary photographic minimum times
if m > 1 plot(E1c,O1c,'k.','MarkerSize',17); end   %primary ccd minimum times
if p > 1 plot(E2v,O2v,'bo','MarkerSize',2); end    %secondary visual minimum times
if o2 > 1 plot(E2pp,O2pp,'bo','MarkerSize',2); end  %secondary photographic minimum times, worse quality
if q > 1 plot(E2p,O2p,'bo','MarkerSize',3); end    %secondary photographic minimum times
if r > 1 plot(E2c,O2c,'bo','MarkerSize',4); end    %secondary ccd minimum times
xlabel('Epoch');   %labeling the axis
ylabel('O-C (Days)');
title(QQ);    %O-C diagram with the LITE curve, another format, better to publish

hold off
%---------------------------------------------------------------------------------------------
clear E1c E1v E1p E1pp E2c E2v E2p E2pp O1v O1c O1p O2v O2c O2p O1pp O2pp j k m n o o1 p q r o2 O;   %clear all unnecessary variables
%---------------------------------------------------------------------------------------------

% ==========  HERE THE WHOLE CYCLE BEGINS: ==========
OuterAbacus=0;

count = input('How many steps in one calculation of the simplex algorithm? \n (a few, depends on the speed of the computer): ');
disp([' ']);
input('Now the numbers on the screen represent (decreasing) sum \nof the square residuals from all minimum times \nin the best vertex of the actual simplex.');

contin = 1;
while contin == 1
    OuterAbacus=OuterAbacus+1;
    %---------------- HERE THE SIMPLEX ALGORITHM!!! ----------------->
    if JD0F == 1
        JD0D=JD00+random('Normal',0,0.011,1,param+1);   %define the vertexes of the initial simplex (x,y,...)=(JD0i,P0i,...):
        JD0D(1)=JD00;
    else
        JD0D=JD00+random('Normal',0,0.0,1,param+1);   %define the vertexes of the initial simplex (x,y,...)=(JD0i,P0i,...):
        JD0D(1)=JD00;
    end
    
    if PF == 1
        PD=P+random('Normal',0,0.000115,1,param+1);
        PD(1)=P;
    else
        PD=P+random('Normal',0,0.0,1,param+1);
        PD(1)=P;
    end

    if excen1F == 1
        excen1D=excen1+random('Normal',0,0.08,1,param+1);
        excen1D(1)=excen1;
        for i = 1:param+1
            if excen1D(i) >= 0.99
                excen1D(i)=0.99;
            end
        end
    else
        excen1D=excen1+random('Normal',0,0.0,1,param+1);
        excen1D(1)=excen1;
    end
    
    if omega1F == 1
        omega1D=omega1+random('Normal',0,115,1,param+1);
        omega1D(1)=omega1;
    else
        omega1D=omega1+random('Normal',0,0,1,param+1);
        omega1D(1)=omega1;
    end
    
    if dodtF == 1
        dodtD=dodt+random('Normal',0,0.125,1,param+1);
        dodtD(1)=dodt;
    else
        dodtD=dodt+random('Normal',0,0.0,1,param+1);
        dodtD(1)=dodt;
    end
    
    konst=400;
    abacus=konst+1;
    fl=0.001;

    for i = 1:konst+1
        SumOfSquares(i)=0;
    end
    
    SumOfSquares(abacus)=1;
  while SumOfSquares(abacus) ~= SumOfSquares(abacus-konst)   %the requirement for terminate the calculations, when the sum of squares is konstant
    abacus=abacus+1;

    %---------- here the values from all data points from all parameters ----------
    for i=1:param+1
         SumOfSquaresAstr(i) = ApsidalMotion(JD0D(i),JDi,Epo0,Weight,N,PD(i),excen1D(i),dodtD(i),omega1D(i));
    end
    %---------------------------------------------------------------------------
    
    Sort0=sort(SumOfSquaresAstr);     %sort the vector from the smallest element -> fh is the highest, fl the smallest
    fh=Sort0(param+1);
    fs=Sort0(param);
    fl=Sort0(1);
    SumOfSquares(abacus)=fl;   %the best, the lowest sum of squares
    
    if mod(abacus,50)==0
        disp(['Sum of square residuals: ', num2str(fl,25), '  Number of iterations: ', int2str(abacus-konst), '/', int2str(OuterAbacus), '/',int2str(count),]);
    end
    
    for i=1:param+1
        if SumOfSquaresAstr(i)==Sort0(param+1)     %gives the number of the worst vertex
            h=i;
        elseif SumOfSquaresAstr(i)==Sort0(param)    %gives the number of the second worst vertex
            s=i;
        else
            if SumOfSquaresAstr(i)==Sort0(1)    %gives the number of the best vertex
                l=i;
            end
        end
    end
    clear SumOfSquaresAstr Sort0;
    
    for i=1:param
        xc(i)=0;
    end
    
    for i=1:param+1                      %calculate the center of the simplex without the worst vertex i<>h
        if i~=h
            xc(1)=xc(1)+1/param*(JD0D(i));
            xc(2)=xc(2)+1/param*(PD(i));
            xc(3)=xc(3)+1/param*(excen1D(i));
            xc(4)=xc(4)+1/param*(omega1D(i));
            xc(5)=xc(5)+1/param*(dodtD(i));
        end
    end
    %      1      2      3         4         5  
    xh=[JD0D(h),PD(h),excen1D(h),omega1D(h),dodtD(h)];     %coordinates of the worst vertex
    xl=[JD0D(l),PD(l),excen1D(l),omega1D(l),dodtD(l)];     %coordinates of the best vertex
    xs=[JD0D(s),PD(s),excen1D(s),omega1D(s),dodtD(s)];     %coordinates of the second worst vertex
    
    x=[JD0D',PD',excen1D',omega1D',dodtD'];    %coordinates of all the 6 vertexes of simplex - in all 5 parametrs
    
    xalfa=(1+alfa)*xc-alfa*xh;    %reflection             !|
%   xbeta=(1-beta)*xc+beta*xh;    %contraction            !|
%   xgama=(1-gama)*xc+gama*xh;    %expansion              !|
    xdelta=xl+delta*(xh-xl);      %shrinkage              !|
%   for beta and gama see lower !

    %------- here the value in the "xalfa" point --------
    SumOfSquaresAstrAlfa = ApsidalMotion(xalfa(1),JDi,Epo0,Weight,N,xalfa(2),xalfa(3),xalfa(5),xalfa(4));
    %------------------------------------------------------------------
   
    %------- here the value in the "xdelta" point --------
    SumOfSquaresAstrDelta = ApsidalMotion(xdelta(1),JDi,Epo0,Weight,N,xdelta(2),xdelta(3),xdelta(5),xdelta(4));
    %------------------------------------------------------------------
    
    if SumOfSquaresAstrAlfa < fl
        xgama=(1-gama)*xc+gama*((1+alfa)*xc-alfa*xh);     %xbeta(xalfa(xh)) !
        %------- here the value in the "xgama" point --------
        SumOfSquaresAstrGama = ApsidalMotion(xgama(1),JDi,Epo0,Weight,N,xgama(2),xgama(3),xgama(5),xgama(4));
        %------------------------------------------------------------------
        if SumOfSquaresAstrGama < SumOfSquaresAstrAlfa
            for i=1:param
                x(h,i)=xgama(i);
            end
            Inp(abacus-konst)=char('g');
        else
            for i=1:param
                x(h,i)=xalfa(i);
            end
            Inp(abacus-konst)=char('a');
        end
           
    else
        if SumOfSquaresAstrAlfa <= fs
            for i=1:param
                x(h,i)=xalfa(i);
            end
             Inp(abacus-konst)=char('a');
         else
             if SumOfSquaresAstrAlfa > fh
                xbeta=(1-beta)*xc+beta*xh;
                %------- here the value in the "xbeta" point --------
                SumOfSquaresAstrBeta = ApsidalMotion(xbeta(1),JDi,Epo0,Weight,N,xbeta(2),xbeta(3),xbeta(5),xbeta(4));
                %------------------------------------------------------------------
            else
                xbeta=(1-beta)*xc+beta*((1+alfa)*xc-alfa*xh);     %xbeta(xalfa(xh)) !
                %------- here the value in the "xbeta" point --------
                SumOfSquaresAstrBeta = ApsidalMotion(xbeta(1),JDi,Epo0,Weight,N,xbeta(2),xbeta(3),xbeta(5),xbeta(4));
                %------------------------------------------------------------------
            end
            if SumOfSquaresAstrBeta > fh
                for j=1:param+1
                   for i=1:param
                       x(j,i)=xl(i)+delta*(x(j,i)-xl(i));
                   end
                end
                Inp(abacus-konst)=char('d');
            else
                for i=1:param
                    x(h,i)=xbeta(i);
                end
                Inp(abacus-konst)=char('b');
            end
        end
    end
    clear JD0D PD excen1D omega1D dodtD;

    for i=1:param+1   %arbitrariness to which vertexes assign which values, in fact we don't know which is now the best and the worst
        JD0D(i)=x(i,1);
        PD(i)=x(i,2);
        if x(i,3) < 0   %negative eccentricities ignore
            excen1D(i)=-x(i,3);
        else
            excen1D(i)=x(i,3);
        end
        omega1D(i)=1/100000000*mod(round((100000000*x(i,4))),36000000000);
        if x(i,5) < 0
            dodtD(i)=-x(i,5);
        else
            dodtD(i)=x(i,5);
        end
    end
end

disp([' ']);
disp('Here are shown the individual simplex steps. a-reflection, b-contraction, d-shrinkage, g-expansion');
disp(Inp);

JD00=JD0D(l);
P=PD(l);
excen1=excen1D(l);
omega1=omega1D(l);
dodt=dodtD(l);

clear JD0D PD excen1D omega1D dodtD;

disp([' ']);
disp(['JD00: ',num2str(JD00,'%8.6f'),' ']);
disp(['Period: ',num2str(P,'%5.9f'),' day']);

disp(['eccentricity: ',num2str(excen1,'%2.5f'),'  ']);
disp(['Omega: ',num2str(omega1,'%4.6f'),' deg']);
disp(['d omega / dt: ',num2str(dodt,'%4.5f'),' deg/yr']);

disp(['Altogether: ',int2str(N),' measurements']);
disp(['The computation ran ', int2str(OuterAbacus), '/',int2str(count)  ' times.']);

if OuterAbacus < count    %the requirement to terminate the whole cycle!!!
    contin = 1;
else
    contin = 0;
end

end   %end of the whole cycle !!

%-------------- to plot a figure: ------------------>
clear O E11 E21 B1 B2 Occ1 Occ2 ;
O = JDi-JD00-P*Epo0;         %O-C difference

% minimumE=1000*floor(min(Epo0)/1000)-1000;
% maximumE=1000*ceil(max(Epo0)/1000)+1000;
minimmaxim=MinMax(O,Weight,N);
minimumO=minimmaxim(1)-0.01;
maximumO=minimmaxim(2)+0.01;
clear minimmaxim;

minJD=minimumE*P+JD00-2400000;
maxJD=maximumE*P+JD00-2400000;
minDate=JulToDate(minJD+2400000);
maxDate=JulToDate(maxJD+2400000);
minPer=minimumO/P;
maxPer=maximumO/P;

NumberPrim=0;
NumberSec=0;
for i = 1:N
   switch round(10*abs(Epo0(i)-round(Epo0(i))))
   case 0               %primary minimum times
       NumberPrim=NumberPrim+1;
       E11(NumberPrim)=Epo0(i);
       Occ1(NumberPrim)=O(i);
       B1(NumberPrim)=Weight(i);
   case 5               %secondary minimum times
       NumberSec=NumberSec+1;
       E21(NumberSec)=Epo0(i);
       Occ2(NumberSec)=O(i);   
       B2(NumberSec)=Weight(i);
   end
end

m=1;
n=1;
o=1;
o1=1;
remove=0;       %some minimum times were eliminated
for i = 1:NumberPrim     %primary minimum times
   switch B1(i)
   case 0          %weight = 0  =>  ignore this minimum!
      remove=remove+1;
   case 1        %visual primary minimum times
      E1v(n)=E11(i);
      O1v(n)=Occ1(i);
      n=n+1;
   case 2        %visual primary minimum times
      E1v(n)=E11(i);
      O1v(n)=Occ1(i);
      n=n+1;
   case 3        %visual primary minimum times
      E1v(n)=E11(i);
      O1v(n)=Occ1(i);
      n=n+1;
   case 4        %visual primary minimum times
      E1v(n)=E11(i);
      O1v(n)=Occ1(i);
      n=n+1;
   case 5       %photographic primary minimum times, worse quality
      E1pp(o1)=E11(i);
      O1pp(o1)=Occ1(i);
      o1=o1+1;
   case 10        %photographic primary minimum times
      E1p(o)=E11(i);
      O1p(o)=Occ1(i);
      o=o+1;
   case 30       %ccd primary minimum times
      E1c(m)=E11(i);
      O1c(m)=Occ1(i);
      m=m+1;
   end
end

p=1;
q=1;
r=1;
o2=1;
for i = 1:NumberSec     %secondary minimum times
   switch B2(i)
   case 0
      remove=remove+1;   %removed secondary minimum times
   case 1         %visual secondary minimum times
      E2v(p)=E21(i);
      O2v(p)=Occ2(i);
      p=p+1;
   case 2         %visual secondary minimum times
      E2v(p)=E21(i);
      O2v(p)=Occ2(i);
      p=p+1;
   case 3         %visual secondary minimum times
      E2v(p)=E21(i);
      O2v(p)=Occ2(i);
      p=p+1;
   case 4         %visual secondary minimum times
      E2v(p)=E21(i);
      O2v(p)=Occ2(i);
      p=p+1;
   case 5       %photographic secondary minimum times, worse quality
      E2pp(o2)=E21(i);
      O2pp(o2)=Occ2(i);
      o2=o2+1;
   case 10         %photographic secondary minimum times
      E2p(q)=E21(i);
      O2p(q)=Occ2(i);
      q=q+1;
   case 30        %ccd secondary minimum times
      E2c(r)=E21(i);
      O2c(r)=Occ2(i);
      r=r+1;
   end
end

i=0;
Pa=P/(1-dodt*pi/(2*pi*180));
clear Epo Tprim Tsec;
for Ep = minimumE:20:maximumE
    i=i+1;
    Epo(i) = Ep;
    Tprim(i) = Primary(JD00,Ep,P,excen1,dodt*pi/180,omega1*pi/180)-JD00-P*Ep;
    Tsec(i)  = Secondary(JD00,Ep,P,excen1,dodt*pi/180,omega1*pi/180)-JD00-P*Ep-Pa/2;
end

%-------------- 
clear CC;
CC=['.txt'];
BB=char(Star);
QQ=sprintf([BB CC]);

OutputData=[Epo; Tprim; Tsec];

BB=char(Star);
%-------------- O U T P U T   DATA FILE --------------->>>
% fid = fopen(QQ,'w');
% fprintf(fid,'%s',BB,': Epoch -- Prim AM -- Sec AM');
% fprintf(fid,'%s\n','');
% fprintf(fid,'%6.5f   %2.6f   %2.6f\n',OutputData);
% status=fclose('all');   
%-------------- 
clear OutputData QQ BB CC fid;

figure(3);
clf;
hold on
plot(Epo,Tprim,'k-');
plot(Epo,Tsec,'b-');
set(gca,'box','on');
if n > 1 plot(E1v,O1v,'k.','MarkerSize',5); end    %plot primary visual minimum times
if o1 > 1 plot(E1pp,O1pp,'k.','MarkerSize',8); end  %primary photographic minimum times, worse quality
if o > 1 plot(E1p,O1p,'k.','MarkerSize',12); end   %primary photographic minimum times
if m > 1 plot(E1c,O1c,'k.','MarkerSize',17); end   %primary ccd minimum times
if p > 1 plot(E2v,O2v,'bo','MarkerSize',2); end    %secondary visual minimum times
if o2 > 1 plot(E2pp,O2pp,'bo','MarkerSize',2); end  %secondary photographic minimum times, worse quality
if q > 1 plot(E2p,O2p,'bo','MarkerSize',3); end    %secondary photographic minimum times
if r > 1 plot(E2c,O2c,'bo','MarkerSize',4); end    %secondary ccd minimum times
set(gca,'FontSize',[12]); 
axis([minimumE maximumE minimumO maximumO]);
set(gca,'XAxisLocation','bottom');
set(gca,'box','off');
set(gca,'YAxisLocation','left');
set(gca,'PlotBoxAspectRatio',[2.4 1 1]);
set(gca,'FontSize',[12]);
xlabel('Epoch', 'FontSize', 12);
ylabel('O-C (days)', 'FontSize', 12);

%RECT=[0.13 0.80503 0.775 0.0001];
RECT=[0.13 0.733603 0.775 0.0001];
axes('position', RECT);
set(gca,'FontSize',[12]);
axis([minDate maxDate -0.01 0.01]);
set(gca,'TickDir','out');
set(gca,'XAxisLocation','top');
set(gca,'XAxisLocation','bottom');

RECT=[0.13 0.733603 0.775 0.0001];
axes('position', RECT);
set(gca,'FontSize',[12]);
axis([minJD+2400000 maxJD+2400000 -0.0001 0.0001]);
set(gca,'XLimMode','manual');
% set(gca,'XTickLabel','2435000|2440000|2445000|2450000|2455000');
% set(gca,'XTick',[ 2435000.0 2440000.0 2445000.0 2450000.0 2455000.0 ]);
if (maxJD-minJD) < 7000 %cca 19 let
    set(gca,'XTick',[ 2417000.0 2418000.0 2419000.0 2420000.0 2421000.0 2422000.0 2423000.0 2424000.0 2425000.0  2426000.0 2427000.0 2428000.0 2429000.0 2430000.0 2431000.0 2432000.0 2433000.0 2434000.0 2435000.0 2436000.0 2437000.0 2438000.0 2439000.0 2440000.0 2441000.0 2442000.0 2443000.0 2444000.0 2445000.0 2446000.0 2447000.0 2448000.0 2449000.0 2450000.0 2451000.0 2452000.0 2453000.0 2454000.0 2455000.0 2456000.0 2457000.0 2458000.0 2459000.0 2460000.0 ]);
    set(gca,'XTickLabel','2417000|2418000|2419000|2420000|2421000|2422000|2423000|2424000|2425000| 2426000|2427000|2428000|2429000|2430000|2431000|2432000|2433000|2434000|2435000|2436000|2437000|2438000|2439000|2440000|2441000|2442000|2443000|2444000|2445000|2446000|2447000|2448000|2449000|2450000|2451000|2452000|2453000|2454000|2455000|2456000|2457000|2458000|2459000|2460000');    
elseif ((maxJD-minJD) >= 7000) & ((maxJD-minJD) < 15000)  %od cca 19 let do 27 let
    set(gca,'XTick',[ 2418000.0 2420000.0 2422000.0 2424000.0 2426000.0 2428000.0 2430000.0 2432000.0 2434000.0 2436000.0 2438000.0 2440000.0 2442000.0 2444000.0 2446000.0 2448000.0 2450000.0 2452000.0 2454000.0 2456000.0 2458000.0 2460000.0 ]);
    set(gca,'XTickLabel',' 2418000|2420000|2422000|2424000|2426000|2428000|2430000|2432000|2434000|2436000|2438000|2440000|2442000|2444000|2446000|2448000|2450000|2452000|2454000|2456000|2458000|2460000');
else
    if ((maxJD-minJD) >= 15000) & ((maxJD-minJD) < 40000)  %od 27 let do cca 95 let
        set(gca,'XTick',[ 2415000.0 2420000.0 2425000.0 2430000.0 2435000.0 2440000.0 2445000.0 2450000.0 2455000.0 2460000.0 ]);
        set(gca,'XTickLabel','2415000|2420000|2425000|2430000|2435000|2440000|2445000|2450000|2455000|2460000');
    else
        set(gca,'XTick',[ 2420000.0 2430000.0 2440000.0 2450000.0 2460000.0 ]);
        set(gca,'XTickLabel','2420000|2430000|2440000|2450000|2460000');
    end
end
set(gca,'TickDir','out');
set(gca,'XAxisLocation','top');

RECT=[0.9046 0.302 0.0001 0.432];
axes('position', RECT);
set(gca,'FontSize',[12]);
axis([-0.01 0.01 minPer maxPer]);
set(gca,'TickDir','in');
set(gca,'YAxisLocation','right');
ylabel('O-C (Period)', 'FontSize', 12);

clear CC;
CC=['.png'];
BB=char(Star);
QQ=sprintf([BB CC]);
% print('-f', '-dpng', QQ);      %transform the O-C diagram into the .png format

clear CCa;
CCa=['.eps'];
QQ=sprintf([BB CCa]);
print('-f', '-depsc', QQ);      %transform the O-C diagram into the .eps format
hold off
%--------------------------------------------------------------------------------------------------------
clear E1c E1v E1p E2c E2v E2p E1pp E2pp O1pp O2pp O1v O1c O1p O2v O2c O2p j k m n o o1 p q r o2 B1 B2 Occ1 Occ2 E11 E21 O;   %clear all unnecessary variables
%--------------------------------------------------------------------------------------------------------
% and also after the subtraction of the Apsidal motion: -------->>

O=JDi-JD00-P*Epo0;
Pa=P/(1-dodt*pi/(2*pi*180));

NumberPrim=0;
NumberSec=0;
for i = 1:N
   switch round(10*abs(Epo0(i)-round(Epo0(i))))
   case 0
       NumberPrim=NumberPrim+1;
       E11(NumberPrim)=Epo0(i);  %primary minimum times
       Occ1(NumberPrim)=O(i)-Primary(JD00,Epo0(i),P,excen1,dodt*pi/180,omega1*pi/180)+JD00+P*Epo0(i);
       B1(NumberPrim)=Weight(i);
   case 5                   %secondary minimum times
       NumberSec=NumberSec+1;
       E21(NumberSec)=Epo0(i);
       Occ2(NumberSec)=O(i)-Secondary(JD00,Epo0(i),P,excen1,dodt*pi/180,omega1*pi/180)+JD00+P*Epo0(i)+Pa/2;;   
       B2(NumberSec)=Weight(i);
   end
end

m=1;
n=1;
o=1;
o1=1;
remove=0;       %some minimum times were eliminated
for i = 1:NumberPrim     %primary minimum times
   switch B1(i)
   case 0          %weight = 0  =>  ignore this minimum!
      remove=remove+1;
   case 1        %visual primary minimum times
      E1v(n)=E11(i);
      O1v(n)=Occ1(i);
      n=n+1;
   case 2        %visual primary minimum times
      E1v(n)=E11(i);
      O1v(n)=Occ1(i);
      n=n+1;
   case 3        %visual primary minimum times
      E1v(n)=E11(i);
      O1v(n)=Occ1(i);
      n=n+1;
   case 4        %visual primary minimum times
      E1v(n)=E11(i);
      O1v(n)=Occ1(i);
      n=n+1;
   case 5       %photographic primary minimum times, worse quality
      E1pp(o1)=E11(i);
      O1pp(o1)=Occ1(i);
      o1=o1+1;
   case 10        %photographic primary minimum times
      E1p(o)=E11(i);
      O1p(o)=Occ1(i);
      o=o+1;
   case 30       %ccd primary minimum times
      E1c(m)=E11(i);
      O1c(m)=Occ1(i);
      m=m+1;
   end
end

p=1;
q=1;
r=1;
o2=1;
for i = 1:NumberSec     %secondary minimum times
   switch B2(i)
   case 0
      remove=remove+1;   %removed secondary minimum times
   case 1         %visual secondary minimum times
      E2v(p)=E21(i);
      O2v(p)=Occ2(i);
      p=p+1;
   case 2         %visual secondary minimum times
      E2v(p)=E21(i);
      O2v(p)=Occ2(i);
      p=p+1;
   case 3         %visual secondary minimum times
      E2v(p)=E21(i);
      O2v(p)=Occ2(i);
      p=p+1;
   case 4         %visual secondary minimum times
      E2v(p)=E21(i);
      O2v(p)=Occ2(i);
      p=p+1;
   case 5       %photographic secondary minimum times, worse quality
      E2pp(o2)=E21(i);
      O2pp(o2)=Occ2(i);
      o2=o2+1;
   case 10         %photographic secondary minimum times
      E2p(q)=E21(i);
      O2p(q)=Occ2(i);
      q=q+1;
   case 30        %ccd secondary minimum times
      E2c(r)=E21(i);
      O2c(r)=Occ2(i);
      r=r+1;
   end
end

clear minimumO maximumO;
minimumO1=Minimum(Occ1,B1,NumberPrim);
minimumO2=Minimum(Occ2,B2,NumberSec);
minimumO=min(minimumO1,minimumO2)-0.01;
maximumO1=Maximum(Occ1,B1,NumberPrim);
maximumO2=Maximum(Occ2,B2,NumberSec);
maximumO=max(maximumO1,maximumO2)+0.01;
minPer=minimumO/P;
maxPer=maximumO/P;

E=minimumE:10:maximumE;
O=0+0.0000000000001*E;

figure(4);
clf;
hold on
set(gca,'box','on');
if n > 1 plot(E1v,O1v,'k.','MarkerSize',5); end    %plot primary visual minimum times
if o1 > 1 plot(E1pp,O1pp,'k.','MarkerSize',8); end  %primary photographic minimum times, worse quality
if o > 1 plot(E1p,O1p,'k.','MarkerSize',12); end   %primary photographic minimum times
if m > 1 plot(E1c,O1c,'k.','MarkerSize',17); end   %primary ccd minimum times
if p > 1 plot(E2v,O2v,'bo','MarkerSize',2); end    %secondary visual minimum times
if o2 > 1 plot(E2pp,O2pp,'bo','MarkerSize',2); end  %secondary photographic minimum times, worse quality
if q > 1 plot(E2p,O2p,'bo','MarkerSize',3); end    %secondary photographic minimum times
if r > 1 plot(E2c,O2c,'bo','MarkerSize',4); end    %secondary ccd minimum times
plot(E,O,'k');
xlabel('Epoch', 'FontSize', 12);   %labeling the axis
axis([minimumE maximumE minimumO maximumO]);
set(gca,'XAxisLocation','bottom', 'FontSize', 12);
set(gca,'box','off');
set(gca,'YAxisLocation','left', 'FontSize', 12);
set(gca,'PlotBoxAspectRatio',[2.4 1 1]);
set(gca,'FontSize',[12]);
ylabel('O-C (days)', 'FontSize', 12);

RECT=[0.13 0.733603 0.775 0.0001];
axes('position', RECT);
set(gca,'FontSize',[14]);
axis([minDate maxDate -0.01 0.01]);
set(gca,'TickDir','out', 'FontSize', 12);
set(gca,'XAxisLocation','top');
set(gca,'XAxisLocation','bottom');

RECT=[0.13 0.733603 0.775 0.0001];
axes('position', RECT);
set(gca,'FontSize',[12]);
axis([minJD+2400000 maxJD+2400000 -0.0001 0.0001]);
set(gca,'XLimMode','manual');
if (maxJD-minJD) < 7000 %cca 19 let
    set(gca,'XTick',[ 2417000.0 2418000.0 2419000.0 2420000.0 2421000.0 2422000.0 2423000.0 2424000.0 2425000.0  2426000.0 2427000.0 2428000.0 2429000.0 2430000.0 2431000.0 2432000.0 2433000.0 2434000.0 2435000.0 2436000.0 2437000.0 2438000.0 2439000.0 2440000.0 2441000.0 2442000.0 2443000.0 2444000.0 2445000.0 2446000.0 2447000.0 2448000.0 2449000.0 2450000.0 2451000.0 2452000.0 2453000.0 2454000.0 2455000.0 2456000.0 2457000.0 2458000.0 2459000.0 2460000.0 ]);
    set(gca,'XTickLabel','2417000|2418000|2419000|2420000|2421000|2422000|2423000|2424000|2425000| 2426000|2427000|2428000|2429000|2430000|2431000|2432000|2433000|2434000|2435000|2436000|2437000|2438000|2439000|2440000|2441000|2442000|2443000|2444000|2445000|2446000|2447000|2448000|2449000|2450000|2451000|2452000|2453000|2454000|2455000|2456000|2457000|2458000|2459000|2460000');    
elseif ((maxJD-minJD) >= 7000) & ((maxJD-minJD) < 15000)  %od cca 19 let do 27 let
    set(gca,'XTick',[ 2418000.0 2420000.0 2422000.0 2424000.0 2426000.0 2428000.0 2430000.0 2432000.0 2434000.0 2436000.0 2438000.0 2440000.0 2442000.0 2444000.0 2446000.0 2448000.0 2450000.0 2452000.0 2454000.0 2456000.0 2458000.0 2460000.0 ]);
    set(gca,'XTickLabel',' 2418000|2420000|2422000|2424000|2426000|2428000|2430000|2432000|2434000|2436000|2438000|2440000|2442000|2444000|2446000|2448000|2450000|2452000|2454000|2456000|2458000|2460000');
else
    if ((maxJD-minJD) >= 15000) & ((maxJD-minJD) < 40000)  %od 27 let do cca 95 let
        set(gca,'XTick',[ 2415000.0 2420000.0 2425000.0 2430000.0 2435000.0 2440000.0 2445000.0 2450000.0 2455000.0 2460000.0 ]);
        set(gca,'XTickLabel','2415000|2420000|2425000|2430000|2435000|2440000|2445000|2450000|2455000|2460000');
    else
        set(gca,'XTick',[ 2420000.0 2430000.0 2440000.0 2450000.0 2460000.0 ]);
        set(gca,'XTickLabel','2420000|2430000|2440000|2450000|2460000');
    end
end
set(gca,'TickDir','out', 'FontSize', 12);
set(gca,'XAxisLocation','top', 'FontSize', 12); 

RECT=[0.9046 0.302 0.0001 0.432];
axes('position', RECT);
set(gca,'FontSize',[14]);
axis([-0.01 0.01 minPer maxPer]);
set(gca,'TickDir','in');
set(gca,'YAxisLocation','right', 'FontSize', 12);
ylabel('O-C (Period)', 'FontSize', 12);

clear CC;
CC=['_Residuals.png'];
BB=char(Star);
QQ=sprintf([BB CC]);
% print('-f', '-dpng', QQ);      %transform the O-C diagram into the .png format

clear CCa;
CCa=['_Residuals.eps'];
QQ=sprintf([BB CCa]);
print('-f', '-depsc', QQ);      %transform the O-C diagram into the .eps format

hold off

BB=char(Star);
CC=['        ; 1=compute, 0=not compute'];
BBa=char(Star);
QQ=sprintf([BBa CC]);
%-------------- O U T P U T --------------->>>
fid = fopen('Parameters.txt','w');
fprintf(fid,'%s\n',QQ');
fprintf(fid,'%9.6f',JD00);
fprintf(fid,'%s','   ');
fprintf(fid,'%1.0f',JD0F);
fprintf(fid,'%s','   ');
fprintf(fid,'%s\n','  HJD_0');
fprintf(fid,'%3.9f',P);
fprintf(fid,'%s','      ');
fprintf(fid,'%1.0f',PF);
fprintf(fid,'%s','   ');
fprintf(fid,'%s\n','  P [days]');
fprintf(fid,'%2.6f',excen1);
fprintf(fid,'%s','         ');
fprintf(fid,'%1.0f',excen1F);
fprintf(fid,'%s','   ');
fprintf(fid,'%s\n','  eccentricity ');
fprintf(fid,'%5.6f',omega1);
fprintf(fid,'%s','       ');
fprintf(fid,'%1.0f',omega1F);
fprintf(fid,'%s','   ');
fprintf(fid,'%s\n','  omega [deg]');
fprintf(fid,'%5.7f',dodt);
fprintf(fid,'%s','        ');
fprintf(fid,'%1.0f',dodtF);
fprintf(fid,'%s','   ');
fprintf(fid,'%s\n','  d omega/dt');
status=fclose('all');

%===========Calculation of the uncertainity of the ephemeris!!!===================
M=2;   %number of parameters to fit (only 2 parameters)
%--------------------------------------------------
yiminusyxi= JDi - (JD00+P*Epo0);   %y_i - y(x_i,JD0,P)
sigmaKvadr=(sum(yiminusyxi.*yiminusyxi))/(N-M);

dydJD0  =  ((JD00+0.0001+P*Epo0) - (JD00+P*Epo0))/0.0001;    %d(OminusC)/dJD0
dydP  =  ((JD00+(P+0.0000001)*Epo0) - (JD00+P*Epo0))/0.0000001;       %d(OminusC)/dP

for k = 1:M
    for l = 1:M
        switch k
            case 1
                first = dydJD0;
            case 2 
                first = dydP;
        end
        switch l
            case 1
                second = dydJD0;
            case 2 
                second = dydP;
        end
        alpha(k,l) = (1/sigmaKvadr)*(sum((first).*(second)));
    end
end

C=inv(alpha);
sigma=sqrt(C);
disp('Absolute errors:');
sigmaJD0=sigma(1,1)
sigmaP=sigma(2,2)

disp(['Relative errors: JD0 => ',num2str(100*sigmaJD0/JD00,'%4.4f'),' %']);
disp(['Relative errors: P => ',num2str(100*sigmaP/P,'%4.4f'),' %']);
clear alpha C sigma M yiminusyxi sigmaKvadr dydJD0 dydP k l first second;
%=========================================================================

%=========== firstly for the primaries: ==========
	M=3;   %number of parameters to fit (only 3 parameters of Apsidal motion)

    yiminusyxiPrim = Primary(JD00,Epo0,P,excen1,dodt*pi/180,omega1*pi/180) - JDi;   %y_i - y(x_i,e,omega,dodt)
    sigmaKvadratPrim = (sum(yiminusyxiPrim.*yiminusyxiPrim))/(N-M);

    dyde  = (Primary(JD00,Epo0,P,excen1+0.00000001,dodt*pi/180,omega1*pi/180) - Primary(JD00,Epo0,P,excen1,dodt*pi/180,omega1*pi/180))/0.00000001;    %d(x)/de
    dydom = (Primary(JD00,Epo0,P,excen1,dodt*pi/180,(omega1+0.00000001)*pi/180) - Primary(JD00,Epo0,P,excen1,dodt*pi/180,omega1*pi/180))/0.00000001;     %d(x)/domega
    dydd  = (Primary(JD00,Epo0,P,excen1,(dodt+0.000000001)*pi/180,omega1*pi/180) - Primary(JD00,Epo0,P,excen1,dodt*pi/180,omega1*pi/180))/0.000000001;   %d(x)/dodt

    for k = 1:M
        for l = 1:M
            switch k
                case 1 
                   first = dyde;
                case 2 
                   first = dydom;
                case 3 
                   first = dydd;
           end
          switch l
              case 1 
                   second = dyde;
              case 2 
                   second = dydom;
              case 3 
                   second = dydd;
           end
           alfaPrim(k,l) = (1/sigmaKvadratPrim)*(sum((first).*(second)));
       end
    end

    CPrim=inv(alfaPrim);
    sigmaPrim=sqrt(CPrim);
    disp('Absolute errors: ');
    %======================================
    clear first second;
    %=========== for secondaries: ==========
    yiminusyxiSec = Secondary(JD00,Epo0,P,excen1,dodt*pi/180,omega1*pi/180) - Pa/2 - JDi; %y_i - y(x_i,e,omega,dodt)
    sigmaKvadratSec = (sum(yiminusyxiSec.*yiminusyxiSec))/(N-M);

    dyde  = (Secondary(JD00,Epo0,P,excen1+0.0000001,dodt*pi/180,omega1*pi/180) - Secondary(JD00,Epo0,P,excen1,dodt*pi/180,omega1*pi/180))/0.0000001;    %d(x)/de
    dydom = (Secondary(JD00,Epo0,P,excen1,dodt*pi/180,(omega1+0.0000001)*pi/180) - Secondary(JD00,Epo0,P,excen1,dodt*pi/180,omega1*pi/180))/0.0000001;     %d(x)/domega
    dydd  = (Secondary(JD00,Epo0,P,excen1,(dodt+0.00000001)*pi/180,omega1*pi/180) - Secondary(JD00,Epo0,P,excen1,dodt*pi/180,omega1*pi/180))/0.00000001;   %d(x)/dodt
    for k = 1:M
        for l = 1:M
            switch k
                case 1 
                   first = dyde;
                case 2 
                   first = dydom;
                case 3 
                   first = dydd;
           end
          switch l
              case 1 
                   second = dyde;
              case 2 
                   second = dydom;
              case 3 
                   second = dydd;
           end
           alfaSec(k,l) = (1/sigmaKvadratSec)*(sum((first).*(second)));
       end
    end

    CSec=inv(alfaSec);
    sigmaSec=sqrt(CSec);
    disp('Absolute errors: ');
    %======================================
    sigmae=(sigmaPrim(1,1) + sigmaSec(1,1))/2
    sigmaom=(sigmaPrim(2,2) + sigmaSec(2,2))/2
    sigmad=(sigmaPrim(3,3) + sigmaSec(3,3))/2

    disp(['Relative errors: e => ',num2str(100*sigmae/excen1,'%4.4f'),' %']);
    disp(['Relative errors: omega => ',num2str(100*sigmaom/omega1,'%4.4f'),' %']);
    disp(['Relative errors: dodt  => ',num2str(100*sigmad/dodt,'%4.4f'),' %']);

%     clear alfa yiminusyxi sigmaKvadrat dydJD0 dydP first second k l C sigma ErrChQ;
    
U=360/(365.24*dodt/P);
Umin=360/(365.24*(dodt+sigmad)/P);
Umax=360/(365.24*(dodt-sigmad)/P);
dU=max(abs(U-Umin),abs(U-Umax));

%-------------- O U T P U T --------------->>>
fid = fopen('Output.txt','w');
fprintf(fid,'%s',BB,': Final parameters');
fprintf(fid,'%s\n','');
fprintf(fid,'%s\n','');
fprintf(fid,'%s','JD0 = ');
fprintf(fid,'%12.7f',JD00);
fprintf(fid,'%s',' +/- ');
fprintf(fid,'%3.7f',sigmaJD0);
fprintf(fid,'%s\n',' HJD');
fprintf(fid,'%s','P = ');
fprintf(fid,'%2.12f',P);
fprintf(fid,'%s',' +/- ');
fprintf(fid,'%3.9f',sigmaP);
fprintf(fid,'%s\n',' days');
fprintf(fid,'%s\n','');
fprintf(fid,'%s','e = ');
fprintf(fid,'%3.5f',excen1);
fprintf(fid,'%s',' +/- ');
fprintf(fid,'%3.5f\n',sigmae);
fprintf(fid,'%s','omega = ');
fprintf(fid,'%5.5f',omega1);
fprintf(fid,'%s',' +/- ');
fprintf(fid,'%3.5f',sigmaom);
fprintf(fid,'%s\n',' deg');
fprintf(fid,'%s','do/dt = ');
fprintf(fid,'%4.5f',dodt);
fprintf(fid,'%s',' +/- ');
fprintf(fid,'%3.5f',sigmad);
fprintf(fid,'%s\n',' deg/cycle');
fprintf(fid,'%s\n','-------------');
fprintf(fid,'%s','U = ');
fprintf(fid,'%4.5f',U);
fprintf(fid,'%s',' +/- ');
fprintf(fid,'%3.5f',dU);
fprintf(fid,'%s\n',' yr');
fprintf(fid,'%s','Pa = ');
fprintf(fid,'%2.12f',Pa);
fprintf(fid,'%s',' +/- ');
fprintf(fid,'%3.9f',sigmaP);
fprintf(fid,'%s\n',' days');
fprintf(fid,'%s\n','-------------');
fprintf(fid,'%s','SUM of the square residuals: ');
fprintf(fid,'%1.13f\n',fl);
status=fclose('all');
%=========================================
